@@ -16,7 +16,7 @@ from indra.databases import hgnc_client, uniprot_client, chebi_client, \
 from indra.statements.resources import amino_acids
 from .term import Term
 from .process import normalize
-from .resources import resource_dir
+from .resources import resource_dir, popular_organisms
 
 
 indra_module_path = indra.__path__[0]
@@ -44,6 +44,7 @@ def generate_hgnc_terms():
     rows = [r for r in read_csv(fname, header=True, delimiter='\t')]
     id_name_map = {r['HGNC ID'].split(':')[1]: r['Approved symbol']
                    for r in rows}
+    organism = '9606'  # human
     for row in rows:
         db, id = row['HGNC ID'].split(':')
         name = row['Approved symbol']
@@ -54,7 +55,7 @@ def generate_hgnc_terms():
             new_id = m.groups()[0]
             new_name = id_name_map[new_id]
             term_args = (normalize(name), name, db, new_id,
-                         new_name, 'previous', 'hgnc')
+                         new_name, 'previous', 'hgnc', organism)
             all_term_args[term_args] = None
             # NOTE: consider adding withdrawn synonyms e.g.,
             # symbol withdrawn, see pex1     symbol withdrawn, see PEX1
@@ -62,12 +63,13 @@ def generate_hgnc_terms():
             continue
         # Handle regular entry official names
         else:
-            term_args = (normalize(name), name, db, id, name, 'name', 'hgnc')
+            term_args = (normalize(name), name, db, id, name, 'name', 'hgnc',
+                         organism)
             all_term_args[term_args] = None
             if row['Approved name']:
                 app_name = row['Approved name']
                 term_args = (normalize(app_name), app_name, db, id, name,
-                             'name', 'hgnc')
+                             'name', 'hgnc', organism)
                 all_term_args[term_args] = None
 
         # Handle regular entry synonyms
@@ -76,7 +78,7 @@ def generate_hgnc_terms():
             synonyms += row['Alias symbols'].split(', ')
         for synonym in synonyms:
             term_args = (normalize(synonym), synonym, db, id, name, 'synonym',
-                         'hgnc')
+                         'hgnc', organism)
             all_term_args[term_args] = None
 
         # Handle regular entry previous symbols
@@ -84,7 +86,7 @@ def generate_hgnc_terms():
             prev_symbols = row['Previous symbols'].split(', ')
             for prev_symbol in prev_symbols:
                 term_args = (normalize(prev_symbol), prev_symbol, db, id, name,
-                             'previous', 'hgnc')
+                             'previous', 'hgnc', organism)
                 all_term_args[term_args] = None
 
     terms = [Term(*args) for args in all_term_args.keys()]
@@ -225,12 +227,14 @@ def generate_famplex_terms(ignore_mappings=False):
         elif 'HGNC' in groundings:
             id = groundings['HGNC']
             term = Term(norm_txt, txt, 'HGNC', hgnc_client.get_hgnc_id(id), id,
-                        'assertion', 'famplex')
+                        'assertion', 'famplex', '9606')
         elif 'UP' in groundings:
             db = 'UP'
             id = groundings['UP']
             name = id
+            organism = None
             if uniprot_client.is_human(id):
+                organism = '9606'
                 hgnc_id = uniprot_client.get_hgnc_id(id)
                 if hgnc_id:
                     name = hgnc_client.get_hgnc_name(hgnc_id)
@@ -239,7 +243,9 @@ def generate_famplex_terms(ignore_mappings=False):
                         id = hgnc_id
                 else:
                     logger.warning('No gene name for %s' % id)
-            term = Term(norm_txt, txt, db, id, name, 'assertion', 'famplex')
+            # TODO: should we add organism info here?
+            term = Term(norm_txt, txt, db, id, name, 'assertion', 'famplex',
+                        organism)
         elif 'CHEBI' in groundings:
             id = groundings['CHEBI']
             name = chebi_client.get_chebi_name_from_id(id[6:])
@@ -263,40 +269,75 @@ def generate_famplex_terms(ignore_mappings=False):
     return terms
 
 
-def generate_uniprot_terms(download=False):
+def generate_uniprot_terms(download=False, organisms=None):
+    if not organisms:
+        organisms = popular_organisms
     path = os.path.join(resource_dir, 'up_synonyms.tsv')
+    org_filter_str = ' OR '.join(organisms)
     if not os.path.exists(path) or download:
-        url = ('https://www.uniprot.org/uniprot/?format=tab&columns=id,'
-               'genes(PREFERRED),protein%20names&sort=score&'
-               'fil=organism:"Homo%20sapiens%20(Human)%20[9606]"'
-               '%20AND%20reviewed:yes')
+        url = (f'https://www.uniprot.org/uniprot/?format=tab&columns=id,'
+               f'genes(PREFERRED),genes(ALTERNATIVE),protein%20names,organism-id&sort=score&'
+               f'query=reviewed:yes&fil=organism:{org_filter_str}')
         logger.info('Downloading UniProt resource file')
         res = requests.get(url)
         with open(path, 'w') as fh:
             fh.write(res.text)
     terms = []
     for row in read_csv(path, delimiter='\t', header=True):
-        names = parse_uniprot_synonyms(row['Protein names'])
         up_id = row['Entry']
-        standard_name = row['Gene names  (primary )']
-        ns = 'UP'
-        id = row['Entry']
+        organism = row['Organism ID']
+        protein_names = parse_uniprot_synonyms(row['Protein names'])
+        primary_gene_name = row['Gene names  (primary )'].strip()
+        if primary_gene_name == ';':
+            primary_gene_name = None
+        gene_synonyms_str = row['Gene names  (synonym )'].strip()
+        if gene_synonyms_str == ';':
+            gene_synonyms_str = None
+        # We generally use the gene name as the standard name
+        # except when there are multiple gene names (separated by
+        # semi-colons) in which case we take the first protein name.
+        if not primary_gene_name or ';' in primary_gene_name:
+            standard_name = protein_names[0]
+        else:
+            standard_name = primary_gene_name
         # We skip a small number of not critical entries that don't have
         # standard names
         if not standard_name:
             continue
+        ns = 'UP'
+        id = up_id
         hgnc_id = uniprot_client.get_hgnc_id(up_id)
         if hgnc_id:
             ns = 'HGNC'
             id = hgnc_id
             standard_name = hgnc_client.get_hgnc_name(hgnc_id)
-        for name in names:
+        for name in protein_names:
             # Skip names that are EC codes
             if name.startswith('EC '):
                 continue
+            if name == standard_name:
+                continue
             term = Term(normalize(name), name, ns, id,
-                        standard_name, 'synonym', 'uniprot')
+                        standard_name, 'synonym', 'uniprot',
+                        organism)
             terms.append(term)
+        term = Term(normalize(standard_name), standard_name,
+                    ns, id, standard_name, 'name', 'uniprot',
+                    organism)
+        terms.append(term)
+        if gene_synonyms_str:
+            # This is to deal with all the variations in which
+            # synonyms are listed, including degenerate strings
+            # like "; ;"
+            for synonym_group in gene_synonyms_str.split('; '):
+                for synonym in synonym_group.split(' '):
+                    if not synonym or synonym == ';':
+                        continue
+                    term = Term(normalize(synonym), synonym,
+                                ns, id, standard_name, 'synonym', 'uniprot',
+                                organism)
+                    terms.append(term)
+
     return terms
 
 
@@ -528,12 +569,12 @@ def get_all_terms():
     terms = []
 
     generated_term_groups = [
+        generate_uniprot_terms(),
         generate_famplex_terms(),
         generate_hgnc_terms(),
         generate_chebi_terms(),
         generate_go_terms(),
         generate_mesh_terms(),
-        generate_uniprot_terms(),
         generate_adeft_terms(),
         generate_doid_terms(),
         generate_hp_terms(),
